######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  %Z%  %M%  %I%  Delta: %G%
######################################################################

globals "globals.4gl"

# Local (static) variables
define
    lookup_prep char(1), # Have the lookups been prepared?
    insert_prep char(1), # Has the insert statement been prepared?
    select_prep char(1), # Has the select statement been prepared?
    exit_level smallint, # 0=input, 1=row, 2=field
    tab_pressed smallint # (boolean) was the tab key pressed?


######################################################################
function lld_input()
# returning -1 if tab pressed (next window), 0 otherwise
######################################################################
#
    define
        goto_top  smallint,  # (boolean) Go to the top of the array?
        in_insert smallint   # (boolean) true if we're in 'insert row'

    # Trap fatal errors
    whenever error call error_handler

    # Set some initial values
    let goto_top = true
        # no detail lines, assume the tab key is pressed
        if rec1_cnt = 0 
        then 
          call fg_err(1)  # no outstanding open items for this customer
          return -1 
        end if 

    # This loop provides the ability to goto the top of the array.
    # Logic contained within the loop is not indented for readability.
    while goto_top

    let goto_top = false
    let in_insert = false
    let exit_level = 0
    let tab_pressed = false
    call set_count(rec1_cnt)
    input array p_rcashd without defaults from s_rcashd.*

      # The following section is for array type inputs
      before row
        if in_insert
        then
            # f1/f2 scenario - fix parallel array if pre-informix 4.0
            if not is_4_0() and rec1_cnt > 0
            then
                call lld_a_delete()
            end if
            let in_insert = false  # Reset in_insert flag
        end if
        let scr_fld = ""           # Reset the current field
        let p_cur = arr_curr()     # Set the current array line
        let s_cur = scr_line()     # Set the current screen line
        let rec1_cnt = arr_count() # Set the number of array elements
        call lld_b_row()
        goto next_field
      after row
        call lld_showline()
        call lld_a_row()
        goto next_field
      before insert
        let in_insert = true
        let rec1_cnt = arr_count() # Set the number of array elements
        call lld_b_insert()
        goto next_field
      after insert
        let in_insert = false
        call lld_a_insert()
        goto next_field
      before delete
        call lld_b_delete()
        goto next_field
      after delete
        call lld_a_delete()
        goto next_field

      # All entry fields must have before and after field processing
      before field pay_method call lld_b_field("formonly.pay_method")
        goto next_field

      after field pay_method call lld_a_field()
        goto next_field

      after input
        label end_input:
        let nxt_fld = null

        # Exit directly upon interrupt
        if int_flag then exit input end if

        # Run the 'after field' logic if necessary
        if exit_level >= 2 then call lld_a_field() end if
        if nxt_fld is not null then goto next_field end if

        # Run the 'after row' logic if necessary
        if exit_level >= 1 then call lld_a_row() end if
        if nxt_fld is not null then goto next_field end if

        # Don't run 'after input' logic if looping to top
        if goto_top then exit input end if

        # Run the 'after input' logic before exiting input
        call lld_a_input()
        if nxt_fld is not null then goto next_field end if
        exit input

      # Event trapping logic
      on key (control-b) let hotkey = 2  goto event
      on key (control-e) let hotkey = 5  goto event
      on key (control-f) let hotkey = 6  goto event
      on key (control-g) let hotkey = 7  goto event
      on key (control-i) let hotkey = 9  goto event
      on key (control-n) let hotkey = 14 goto event
      on key (control-o) let hotkey = 15 goto event
      on key (control-p) let hotkey = 16 goto event
      on key (control-t) let hotkey = 20 goto event
      on key (control-u) let hotkey = 21 goto event
      on key (control-v) let hotkey = 22 goto event
      on key (control-w) let hotkey = 23 goto event
      on key (control-y) let hotkey = 25 goto event
      on key (control-z) let hotkey = 26 goto event
      on key (f5)  let hotkey = 105 goto event
      on key (f6)  let hotkey = 106 goto event
      on key (f7)  let hotkey = 107 goto event
      on key (f8)  let hotkey = 108 goto event
      on key (f9)  let hotkey = 109 goto event
      on key (f10) let hotkey = 110 goto event
      on key (f11) let hotkey = 111 goto event
      on key (f12) let hotkey = 112 goto event
      on key (f13) let hotkey = 113 goto event
      on key (f14) let hotkey = 114 goto event
      on key (f15) let hotkey = 115 goto event
      on key (f16) let hotkey = 116 goto event
      on key (f34) let hotkey = 134 goto event
      on key (f35) let hotkey = 135 goto event
      on key (f36,interrupt) let hotkey = 136 goto event

      # Local event processing
      label event:
        call hot_local("cancel")
        call hot_key("lld_input")  # Map the key to the event
        call lld_event()

      # Programmatic next field logic
      label next_field:
        case
          when nxt_fld is null  # No need to go through 'case'
          when nxt_fld = "pay_method" next field pay_method
          when nxt_fld = "exit input" goto end_input
          when nxt_fld = "goto top"
            call mld_arr_count()
            let goto_top = true
            goto end_input
        end case

    end input

    # This is the end of the 'goto top' loop.
    end while

    # Return -1 if tab was pressed
    if tab_pressed
      then return -1
      else return 0
    end if
end function
# lld_input()

######################################################################
function lld_b_field(field_name)
######################################################################
# This function is called from the input function before every field.
# The 'prv_fld' variable contains the field we came from.
# The 'scr_fld' variable contains the field we're going into.
# Set 'nxt_fld' if you want to skip this field or exit input.
#
    define
        field_name char(40),  # Fieldname passed
        prv_fld char(40)      # The field we were just in

    # Convert field_name from 'tab.col' to 'col'
    let field_name = set_fldtab(field_name)

    # Rearrange field names
    let prv_fld = scr_fld
    let scr_fld = field_name
    let nxt_fld = null

    # Require field and row level exit
    let exit_level = 2

    # Setup for lib_before
    let scr_fld = prv_fld
    call lib_before(field_name)

end function
# lld_b_field()

######################################################################
function lld_a_field()
######################################################################
# This function is called after every field.
#
    # Reset nxt_fld
    let nxt_fld = null

    # Require row level exit
    let exit_level = 1

    # This (among other things) sets data_changed
    call lib_after()

    # After data_changed logic
    if data_changed
    then
        case
          when scr_fld = "pay_method"
            if p_rcashd[p_cur].pay_method matches "[iI]"
            then
                let scratch =
                  " stropend.doc_no = \"", q_rcashd[p_cur].inv_doc_no, "\""
                if zoom("arinvcz","(see scratch)") then end if
                let p_rcashd[p_cur].pay_method = null
                let nxt_fld = "pay_method"
                return
            end if
            if p_rcashd[p_cur].pay_method matches "[cC]"
            then
                call cust_detail(m_rcashe.cust_code)
                let p_rcashd[p_cur].pay_method = null
                let nxt_fld = "pay_method"
            end if
            if p_rcashd[p_cur].pay_method < "1" or
               p_rcashd[p_cur].pay_method > "4"
            then
                call fg_err(10) # Payment method must be between 1 and 4
                let p_rcashd[p_cur].pay_method = null
                let nxt_fld = "pay_method"
            end if
            call pay_ar_line(p_cur,p_rcashd[p_cur].pay_method)
            let p_rcashd[p_cur].pay_method = null
            display p_rcashd[p_cur].* to s_rcashd[s_cur].*
            display by name p_rcashe.paid_tot, p_rcashe.disc_tot,
                    p_rcashe.bal_remaining
        end case
    end if

end function
# lld_a_field()

######################################################################
function lld_a_input()
######################################################################
# This function is called whenever the input statement exits
# (except due to an interrupt).  If you don't want the input session
# to end, set the nxt_fld variable to contain the field to be placed
# back into.
#

    if tab_pressed = false
      then call cust_unlock() end if
end function
# lld_a_input()

######################################################################
function lld_event()
######################################################################
# This function is called whenever the user presses an event key.
# The event is mapped to the 'scr_funct' variable and processed here.
#
    # Reset tab pressed to false
    let tab_pressed = false
    case
      when scr_funct = "cancel"
        call cust_unlock()
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
      when scr_funct = "accept"
        let nxt_fld = "exit input"
      when scr_funct = "tab" or scr_funct = "btab"
        let tab_pressed = true
        let nxt_fld = "exit input"
      when scr_funct = "cancel"
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
    end case
    let scr_funct = ""

end function
# lld_event()

######################################################################
function lld_b_row()
######################################################################
# This function is called before you enter a new row.
#

end function
# lld_b_row()

######################################################################
function lld_a_row()
######################################################################
# This function is called whenever you leave a row.
#
    # No more exit levels required
    let exit_level = 0

end function
# lld_a_row()

######################################################################
function lld_b_insert()
######################################################################
# This function is called before a new row is added to the array.
#
    define
        n smallint   # Generic counter

    # Expand (shift) the parallel array
    for n = (rec1_cnt - 1) to p_cur step -1
        let q_rcashd[n+1].* = q_rcashd[n].*
    end for

    # Blank out the current array element
    initialize q_rcashd[p_cur].* to null

    let nxt_fld = "goto top"
    return
end function
# lld_b_insert()

######################################################################
function lld_a_insert()
######################################################################
# This function is called after a new row has been added to the array.
#

end function
# lld_a_insert()

######################################################################
function lld_b_delete()
######################################################################
# This function is called when the user presses [F2] (delete a row),
# and before the actual array elements have been shifted.
#
    let p_rcashe.disc_tot = p_rcashe.disc_tot - p_rcashd[p_cur].disc_amt
    let p_rcashe.paid_tot = p_rcashe.paid_tot - p_rcashd[p_cur].paid
    let p_rcashe.bal_remaining = p_rcashe.bal_remaining +
        p_rcashd[p_cur].paid - p_rcashd[p_cur].disc_amt
    display by name p_rcashe.disc_tot,
                    p_rcashe.paid_tot, p_rcashe.bal_remaining

end function
# lld_b_delete()

######################################################################
function lld_a_delete()
######################################################################
# This function is called when the user presses [F2] (delete a row),
# and after the actual array elements have been shifted.
# (rec1_cnt contains the number of elements before the delete)
#
    define
        n smallint   # Generic counter

    # Compress (shift) the parallel array
    for n = (p_cur + 1) to rec1_cnt
        let q_rcashd[n-1].* = q_rcashd[n].*
    end for

    # Blank out the last (duplicate) data element
    initialize p_rcashd[rec1_cnt].* to null
    initialize q_rcashd[rec1_cnt].* to null

    # Perform array math
    call llh_math()
    call llh_display()

end function
# lld_a_delete()

######################################################################
function lld_setdata()
######################################################################
# This function is called to set the this_data variable
#
    case
      when scr_fld = "pay_method"
        call set_data(p_rcashd[p_cur].pay_method)
    end case
end function
# lld_setdata()

######################################################################
function lld_high()
######################################################################
# This function highlights the specified field name.
# Only input type fields need to be specified.
#
    case
      when scr_fld = "pay_method"
        display p_rcashd[p_cur].pay_method
          to s_rcashd[s_cur].pay_method attribute(reverse)
    end case
end function
# lld_high()

######################################################################
function lld_showline()
######################################################################
# This function displays the current p_* variables onto the screen line
#
    display p_rcashd[p_cur].* to s_rcashd[s_cur].*
end function
# lld_showline()

######################################################################
function lld_display()
######################################################################
# This function displays the first screenful of data
#
    for p_cur = 1 to scr1_max
        display p_rcashd[p_cur].* to s_rcashd[p_cur].*  attribute(red)
    end for
end function
# lld_display()

######################################################################
function lld_p_prep(n)
######################################################################
# This function creates the p_[]* record from the m_* record
#
    define
        n smallint      # Array element to process


    let q_rcashd[n].inv_doc_no = m_rcashd.inv_doc_no
    let q_rcashd[n].dist_acct = m_rcashd.dist_acct
    let q_rcashd[n].dist_department = m_rcashd.dist_department
    select * into m_ropend.* from stropend
        where stropend.doc_no = m_rcashd.inv_doc_no
    if sqlca.sqlcode = notfound
    then
        let m_ropend.disc_bal = 0
        let m_ropend.balance = 0
    end if
    let p_rcashd[n].inv_no = m_ropend.inv_no
    let q_rcashd[n].inv_doc_no = m_ropend.doc_no
    let p_rcashd[n].due_date = m_ropend.due_date
    let p_rcashd[n].disc_date = m_ropend.disc_date
    let p_rcashd[n].disc_bal = m_ropend.disc_bal - m_rcashd.disc_amt
    if m_rcashd.dist_deb_cred = "DB"
    then
        let p_rcashd[n].balance = m_ropend.balance + m_rcashd.dist_amt
    else
        let p_rcashd[n].balance = m_ropend.balance - m_rcashd.dist_amt
    end if
    let m_rcashd.dist_acct = m_ropend.ar_acct_no
    let m_rcashd.dist_department = m_ropend.ar_department
    let p_rcashd[n].pay_method = null
    if m_rcashd.dist_amt is null
      then let m_rcashd.dist_amt = 0 end if
    if m_rcashd.disc_amt is null
      then let m_rcashd.disc_amt = 0 end if
    let q_rcashd[n].dist_acct = m_rcashd.dist_acct
    let q_rcashd[n].dist_department = m_rcashd.dist_department
    let p_rcashd[n].paid = m_rcashd.dist_amt
    if m_rcashd.dist_deb_cred = "DB"
    then
        let p_rcashd[n].paid = m_rcashd.dist_amt * -1
    end if
    let p_rcashd[n].disc_amt = m_rcashd.disc_amt
    if m_rcashd.disc_deb_cred = "CR"
    then
        let p_rcashd[n].disc_amt = m_rcashd.disc_amt * -1
    end if
    let p_rcashe.disc_tot = p_rcashe.disc_tot + p_rcashd[n].disc_amt
    let p_rcashe.paid_tot = p_rcashe.paid_tot + p_rcashd[n].disc_amt

end function
# lld_p_prep()

######################################################################
function lld_m_prep(n)
######################################################################
# This function creates the m_* record from the specified element
# of the p_.* array, and from elements of the header that tie the
# detail lines to the header
#
    define
        n smallint      # Array element to process

    # Items from the header...
    let m_rcashd.doc_no = m_rcashe.doc_no

    # Items from the p_ record array...
    let m_rcashd.inv_doc_no = q_rcashd[n].inv_doc_no
    let m_rcashd.dist_acct = q_rcashd[n].dist_acct
    let m_rcashd.dist_department = q_rcashd[n].dist_department
    let m_rcashd.inv_no = p_rcashd[n].inv_no
    let m_rcashd.due_date = p_rcashd[n].due_date
    let m_rcashd.disc_amt = p_rcashd[n].disc_amt
    let m_rcashd.disc_acct = m_rcntrc.cr_disc_acct_no
    let m_rcashd.disc_department = m_rcashd.dist_department
    let m_rcashd.disc_deb_cred = "DB"
    if p_rcashd[n].paid is not null
    then
        let m_rcashd.dist_deb_cred = "CR"
        let m_rcashd.dist_amt = p_rcashd[n].paid
        if p_rcashd[n].paid < 0
        then
            let m_rcashd.dist_amt = p_rcashd[n].paid * -1
            let m_rcashd.dist_deb_cred = "DB"
        end if
    end if
    if m_rcashd.disc_amt < 0
    then
        let m_rcashd.disc_amt = m_rcashd.disc_amt * -1
        let m_rcashd.disc_deb_cred = "CR"
    end if

end function
# lld_m_prep()

######################################################################
function lld_delete()
######################################################################
# This function deletes the rows in the detail table
# that match the where clause
#
    delete from strcashd
      where strcashd.doc_no = m_rcashe.doc_no
end function
# lld_delete()


######################################################################
function lld_read()
######################################################################
# This function reads in the array elements from disk.
# It first reads the data into the m_* records, then calls
# p_prep for each array element.
#
    define
        row_id integer  # Rowid of the fetched element

    # Prepare the cursor upon first invocation
    if select_prep is null
    then
        let select_prep = "Y"
        let scratch = "select rowid, * from strcashd ",
          "where strcashd.doc_no = ? order by due_date, inv_doc_no"
        prepare read_specs from scratch
        declare d_cursor cursor for read_specs
    end if

    # Read in the detail lines
    let p_cur = 0
    open d_cursor using m_rcashe.doc_no
    foreach d_cursor into row_id, m_rcashd.*
        let p_cur = p_cur + 1
        if p_cur > rec1_max
        then
            let sqlca.sqlcode = -1
            return
        end if
        call lld_p_prep(p_cur)
        let q_rcashd[p_cur].row_id = row_id
    end foreach
    close d_cursor
    let rec1_cnt = p_cur
    call set_count(rec1_cnt)
end function
# lld_read()

######################################################################
function lld_add()
######################################################################
# This function inserts data into the detail table.
#
    # Declare an insert cursor for maximum efficiency
    if insert_prep is null
    then
        let insert_prep = "Y"
        declare d_insert cursor for
          insert into strcashd values(m_rcashd.*)
    end if

    # Insert all array elements
    open d_insert
    for p_cur = 1 to rec1_cnt
        call lld_m_prep(p_cur)
        if m_rcashd.dist_amt != 0 or
           m_rcashd.disc_amt != 0
        then
            put d_insert
        end if
    end for
    close d_insert

end function
# lld_add()
