########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ar.4gm/i_arcshe.4gs/fg_funcs.4gl  1.2  Delta: 2/14/91
########################################################################

globals "globals.4gl"


#######################################################################
function dup_cust()
#   returns true/false based upon dup/no dup
#######################################################################
#   - This checks if this customer has already been entered.
#
    define
        scratch integer, # something to select into
        scratch1 integer # temporary doc_no

    if m_rcashe.doc_no is null
    then
        let scratch1 = 0
    else
        let scratch1 = m_rcashe.doc_no
    end if

    select count(*) into scratch from strcashe
        where cust_code = p_rcashe.cust_code
            and doc_no != scratch1
    if scratch = 0  # no duplicate is found
    then
        return false
    else
        return true
    end if

end function
# dup_cust()

#######################################################################
function automatic_pay()
#######################################################################
# pays the on account balance first, then scans through the invoice
# lines and pays those that have a balance left in them.  any money
# left over is applied to "on account".
#
    define
        n integer # generic counter

    if p_rcashe.bal_remaining < 0
    then
        let p_rcashe.bal_remaining = 0
    end if
    for n = 1 to rec1_cnt
        if p_rcashd[n].balance is null or p_rcashd[n].paid != 0
        then
            continue for
        end if
        let p_rcashd[n].balance = p_rcashd[n].balance + p_rcashd[n].paid
        let p_rcashd[n].paid = 0
        let p_rcashd[n].disc_bal = p_rcashd[n].disc_bal +
                                   p_rcashd[n].disc_amt
        let p_rcashd[n].disc_amt = 0
        call pay_ar_line(n,1)
    end for
    let oa_amt = p_rcashe.rcpt_amt - p_rcashe.paid_tot +
        p_rcashe.disc_tot
    let p_rcashe.bal_remaining = 0
    call llh_display()
    call lld_display()
end function
# automatic_pay

#######################################################################
function new_cust()
#######################################################################
# get a new customer's information and open items
#
    select ar_acct_dflt, ar_department_dflt into
        oa_acct, oa_department from strcustr
        where cust_code = p_rcashe.cust_code
    if sqlca.sqlcode = notfound
    then
        call fg_err(8) # Invalid customer code
        let sqlca.sqlcode = 0
        return false
    end if
    if not cust_lock(p_rcashe.cust_code)
    then
        call fg_err(11) # "Cannot lock customer record."
        return false
    end if
    call init_cust_header()
    call llh_display()
    call init_cust_detail(0)
    if sqlca.sqlcode = -1
    then
        call fg_err(9) # Too many invoices in stropend to read
        let sqlca.sqlcode = 0
        let int_flag = -1
        return false
    end if
    call lld_display()
    return true
end function
# new_cust()

#######################################################################
function init_cust_header()
#######################################################################
# sets up all the header amounts for a new or changed customer

    let m_rcashe.cust_code = p_rcashe.cust_code
    if p_rcashe.rcpt_amt is null
    then
        let p_rcashe.rcpt_amt = 0
    end if
    if p_rcashe.on_acct_amt is null
    then
        let p_rcashe.on_acct_amt = 0
    end if
    call calc_on_acct()
    let p_rcashe.disc_tot = 0
    let p_rcashe.paid_tot = 0
    let oa_amt = 0
    let p_rcashe.bal_remaining = p_rcashe.rcpt_amt - p_rcashe.on_acct_amt
    if oa_acct is null
    then
        let oa_acct = m_rcntrc.ar_acct_no
    end if
    if oa_department is null
    then
        let oa_department = "000"
    end if
end function
# init_cust_header

#######################################################################
function init_cust_detail(n)
#######################################################################
#   - this function reads all the customer's non-zero invoices into
#     the p_rcashd array
#   - it uses a cursor, but doesn't lock any rows.
#   - it reads into the m_* record, and moves each item to the p_* array
#
    define
        n           smallint,   # working number
        scratch_date date

    if n = 0
    then
        call mld_clear()
    end if

    let m_rcashd.dist_amt = 0
    let m_rcashd.disc_amt = 0

    whenever error continue
    declare e_cursor cursor for
        select doc_no, due_date from stropend where
            stropend.cust_code = p_rcashe.cust_code and
            stropend.balance != 0 and
            stropend.doc_no not in 
              (select inv_doc_no from strcashe, strcashd where
                 strcashe.cust_code = p_rcashe.cust_code and 
                 strcashe.doc_no = strcashd.doc_no)
            order by stropend.due_date, stropend.doc_no

    foreach e_cursor into m_rcashd.inv_doc_no, scratch_date
        let n = n + 1
        if n > rec1_max
        then
            let n = 0
            let sqlca.sqlcode = -1
            exit foreach
        end if
        call lld_p_prep(n)            # move data from m_* to p_[].*
    end foreach

    whenever error call error_handler
    let rec1_cnt = n
    return
end function
# init_cust_detail()

#######################################################################
function calc_on_acct()
# returning the calculated on account amount
#######################################################################
# this function calculates the actual on account amount by total
# all similar cash receipts's applied on account amount
#
    define
        db_oa_amt like strcustr.on_acct_amt,
        cr_oa_amt like strcustr.on_acct_amt

    if m_rcashe.doc_no is null then let m_rcashe.doc_no = 0 end if

    select sum(strcashe.oa_amt) into db_oa_amt from strcashe where
        strcashe.cust_code = p_rcashe.cust_code and
        strcashe.doc_no != m_rcashe.doc_no and
        strcashe.oa_deb_cred = "DB"

    select sum(strcashe.oa_amt) into cr_oa_amt from strcashe where
        strcashe.cust_code = p_rcashe.cust_code and
        strcashe.doc_no != m_rcashe.doc_no and
        strcashe.oa_deb_cred = "CR"

    if db_oa_amt is null then let db_oa_amt = 0 end if
    if cr_oa_amt is null then let cr_oa_amt = 0 end if

    let p_rcashe.on_acct_amt = p_rcashe.on_acct_amt + db_oa_amt - cr_oa_amt

end function
# calc_on_acct

#######################################################################
function pay_ar_line(n,pay_type)
#######################################################################
# pays an invoice line based on the pay type as follows:
#   1) auto pay.  take discount if within date.  pay all or whats left
#   2) take discount.  like 1 except discount always taken
#   3) no discount.  like 1 except discount never taken
#   4) manual.  ask discount and amount.
#
    define
        n smallint,
        pay_type char(1),
        old_pay_amt like strcashd.dist_amt,
        pay_amt like strcashd.dist_amt,
        old_discount like strcashd.disc_amt,
        discount like strcashd.disc_amt,
        prompt_response char(1)

    if p_rcashd[n].balance is null or pay_type is null or pay_type = " "
    then
        return
    end if
    let old_pay_amt = p_rcashd[n].paid - p_rcashd[n].disc_amt
    let p_rcashd[n].balance = p_rcashd[n].balance + p_rcashd[n].paid
    let p_rcashd[n].paid = 0
    let old_discount = p_rcashd[n].disc_amt
    let p_rcashd[n].disc_bal = p_rcashd[n].disc_bal +
                               p_rcashd[n].disc_amt
    let p_rcashd[n].disc_amt = 0
    let pay_amt = p_rcashd[n].balance
    let discount = 0
    let p_rcashe.bal_remaining = p_rcashe.bal_remaining + old_pay_amt
    let p_rcashe.disc_tot = p_rcashe.disc_tot - old_discount
    let p_rcashe.paid_tot = p_rcashe.paid_tot -
                            old_pay_amt - old_discount

    if (p_rcashd[n].disc_date >= p_rcashe.rcpt_date and
        pay_type = "1") or
        pay_type = "2"
    then
        let discount = p_rcashd[n].disc_bal
        let pay_amt = pay_amt - discount
    end if

    if pay_amt > p_rcashe.bal_remaining
    then
        let pay_amt = p_rcashe.bal_remaining
        if ((p_rcashd[n].disc_date >= p_rcashe.rcpt_date and
             pay_type = "1") or
             pay_type = "2") and
             discount != 0
        then
            let discount =  round( "a",
                pay_amt * discount / (p_rcashd[n].balance - discount) )
        end if
    end if

    if pay_type = "4"
    then

        while prompt_response is null
          prompt "Enter the amount to apply: " for pay_amt
          if int_flag != 0
          then
              let int_flag = 0
              return
          end if

          if (p_rcashe.bal_remaining - pay_amt) < 0
          then
            open window end_setup at 14,16 with 8 rows, 51 columns
                attribute (border, blue)
            let prompt_response = null
            display "===================================================" at 3,1
            display " Warning: The total applied amount is greater      " at 4,1
            display " than the receipt amount.  If you wish to continue," at 5,1
            display " the surplus amount will be applied to the on      " at 6,1
            display " account amount.                                   " at 7,1
            display "===================================================" at 8,1
            while prompt_response matches "[^yYnN]" or
                  prompt_response is null
                prompt " Continue? " for char prompt_response
            end while
            close window end_setup
            if prompt_response = "N" or prompt_response = "n"
            then
                return
            end if
          end if

          if prompt_response is null
          then
              exit while
          end if

        end while

        prompt "Enter the amount of discount to apply: " for discount
        if int_flag != 0
        then
            let int_flag = 0
            return
        end if
        if pay_amt is null
        then
            let pay_amt = 0
        end if
        if discount is null
        then
            let discount = 0
        end if
    end if
    let p_rcashe.bal_remaining = p_rcashe.bal_remaining - pay_amt
    let p_rcashe.disc_tot = p_rcashe.disc_tot + discount
    let p_rcashe.paid_tot = p_rcashe.paid_tot + pay_amt + discount
    let p_rcashd[n].paid = p_rcashd[n].paid + pay_amt + discount
    let p_rcashd[n].disc_amt = p_rcashd[n].disc_amt + discount
    let p_rcashd[n].balance = p_rcashd[n].balance - pay_amt - discount
    let p_rcashd[n].disc_bal = p_rcashd[n].disc_bal - discount
end function
# pay_ar_line(n,pay_type)

##################################################################
function ok_update()
# returning true or false based upon ok to update
##################################################################
#
    if not cust_lock(m_rcashe.cust_code)
    then
             call fg_err(11) # "Cannot lock customer row."
             return false
    end if
    call please_wait()
    call init_cust_detail(rec1_cnt)
    call lld_display()
    return true
end function
# ok_update
