######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  %Z%  %M%  %I%  Delta: %G%
######################################################################

globals "globals.4gl"

# Local (static) variables
define
    lookup_prep char(1), # Have the lookups been prepared?
    select_prep char(1), # Has the select statement been prepared?
    exit_level smallint, # 0=input, 1=field
    tab_pressed smallint # (boolean) was the tab key pressed?


######################################################################
function llh_input()
# returning -1 if tab pressed (next window), 0 otherwise
######################################################################
#
    # Trap fatal errors
    whenever error call error_handler

    # Initialize some variables
    let exit_level = 0
    let tab_pressed = false


    input p_rcashe.* without defaults from s_rcashe.*

      # All entry fields must have before and after field processing
      before field rcpt_date call llh_b_field("strcashe.rcpt_date")
        goto next_field
      before field cust_code call llh_b_field("strcashe.cust_code")
        goto next_field
      before field check_no call llh_b_field("strcashe.check_no")
        goto next_field
      before field doc_desc call llh_b_field("strcashe.doc_desc")
        goto next_field
      before field rcpt_amt call llh_b_field("formonly.rcpt_amt")
        goto next_field
      before field auto_pay call llh_b_field("formonly.auto_pay")
        goto next_field
      before field cash_acct call llh_b_field("strcashe.cash_acct")
        goto next_field
      before field cash_department call llh_b_field("strcashe.cash_department")
        goto next_field

      after field rcpt_date call llh_a_field()
        goto next_field
      after field cust_code call llh_a_field()
        goto next_field
      after field check_no call llh_a_field()
        goto next_field
      after field doc_desc call llh_a_field()
        goto next_field
      after field rcpt_amt call llh_a_field()
        goto next_field
      after field auto_pay call llh_a_field()
        goto next_field
      after field cash_acct call llh_a_field()
        goto next_field
      after field cash_department call llh_a_field()
        goto next_field

      after input
        label end_input:
        let nxt_fld = null

        # Exit directly upon interrupt
        if int_flag then exit input end if

        # Run the 'after field' logic if necessary
        if exit_level >= 1 then call llh_a_field() end if
        if nxt_fld is not null then goto next_field end if

        # Run the 'after input' logic before exiting input
        call llh_a_input()
        if nxt_fld is not null then goto next_field end if
        exit input

      # Event trapping logic
      on key (control-b) let hotkey = 2  goto event
      on key (control-e) let hotkey = 5  goto event
      on key (control-f) let hotkey = 6  goto event
      on key (control-g) let hotkey = 7  goto event
      on key (control-i) let hotkey = 9  goto event
      on key (control-n) let hotkey = 14 goto event
      on key (control-o) let hotkey = 15 goto event
      on key (control-p) let hotkey = 16 goto event
      on key (control-t) let hotkey = 20 goto event
      on key (control-u) let hotkey = 21 goto event
      on key (control-v) let hotkey = 22 goto event
      on key (control-w) let hotkey = 23 goto event
      on key (control-y) let hotkey = 25 goto event
      on key (control-z) let hotkey = 26 goto event
      on key (f5)  let hotkey = 105 goto event
      on key (f6)  let hotkey = 106 goto event
      on key (f7)  let hotkey = 107 goto event
      on key (f8)  let hotkey = 108 goto event
      on key (f9)  let hotkey = 109 goto event
      on key (f10) let hotkey = 110 goto event
      on key (f11) let hotkey = 111 goto event
      on key (f12) let hotkey = 112 goto event
      on key (f13) let hotkey = 113 goto event
      on key (f14) let hotkey = 114 goto event
      on key (f15) let hotkey = 115 goto event
      on key (f16) let hotkey = 116 goto event
      on key (f34) let hotkey = 134 goto event
      on key (f35) let hotkey = 135 goto event
      on key (f36,interrupt) let hotkey = 136 goto event

      # Local event processing
      label event:
        call hot_local("cancel")
        call hot_key("llh_input")  # Map the key to the event
        call llh_event()

      # Programmatic next field logic
      label next_field:
        case
          when nxt_fld is null  # No need to go through 'case'
          when nxt_fld = "rcpt_date" next field rcpt_date
          when nxt_fld = "cust_code" next field cust_code
          when nxt_fld = "check_no" next field check_no
          when nxt_fld = "doc_desc" next field doc_desc
          when nxt_fld = "rcpt_amt" next field rcpt_amt
          when nxt_fld = "auto_pay" next field auto_pay
          when nxt_fld = "cash_acct" next field cash_acct
          when nxt_fld = "cash_department" next field cash_department
          when nxt_fld = "exit input" goto end_input
        end case
    end input

    # Return -1 if tab was pressed
    if tab_pressed
      then return -1
      else return 0
    end if
end function
# llh_input()

######################################################################
function llh_b_field(field_name)
######################################################################
# This function is called from the input function before every field.
# The 'prv_fld' variable contains the field we came from.
# The 'scr_fld' variable contains the field we're going into.
# Set 'nxt_fld' if you want to skip this field or exit input.
#
    define
        field_name char(40),  # Fieldname passed
        prv_fld char(40)      # The field we were just in

    # Convert field_name from 'tab.col' to 'col'
    let field_name = set_fldtab(field_name)

    # Rearrange field names
    let prv_fld = scr_fld
    let scr_fld = field_name
    let nxt_fld = null

    # Require field level exit
    let exit_level = 1

    # Specify fields only if work needs to be done.
    case
      when scr_fld = "cust_code"
    let scratch = "zoom"
      when scr_fld = "cash_acct"
    let scratch = "zoom"
      when scr_fld = "cash_department"
    let scratch = "zoom"
    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # Setup for lib_before
    let scr_fld = prv_fld
    call lib_before(field_name)

end function
# llh_b_field()

######################################################################
function llh_a_field()
######################################################################
# This function is called after every field.
#
    # Reset nxt_fld
    let nxt_fld = null

    # No more exit levels required
    let exit_level = 0

    # After field logic
    case
      when scr_fld = "rcpt_date"
        if p_rcashe.rcpt_date is null
          then let p_rcashe.rcpt_date = today end if
      when scr_fld = "cust_code"
        if auto_zoom("custz","strcustr","")
          then let p_rcashe.cust_code = scratch end if
      when scr_fld = "rcpt_amt"
        if p_rcashe.rcpt_amt is null
          then let p_rcashe.rcpt_amt = 0 end if
      when scr_fld = "cash_department"
        if p_rcashe.cash_department is null
          then let p_rcashe.cash_department = "000" end if
    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # This (among other things) sets data_changed
    call lib_after()

    # After data_changed logic
    if data_changed
    then
        case
          when scr_fld = "rcpt_date"
            # make sure g/l is installed
            if table_exist("stgcntrc") 
            then
                # check to see if document date is within
                #   current period
                if per_warn(p_rcashe.rcpt_date) = true
                then
                   # Warning:  Date Outside Current Period.
                   call fg_err(13)
                end if
            end if
          when scr_fld = "cust_code"
            # Perform lookups
            if llh_lookup("cust_look",true) = false and
              length(this_data) != 0
            then
                let nxt_fld = "cust_code"
                return
            end if
            if not new_cust()
            then
                let p_rcashe.cust_code = null
                if int_flag != 0
                then
                    let nxt_fld = "exit input"
                    return
                end if
                let nxt_fld = "cust_code"
                return
            end if
            let p_rcashe.cash_acct = m_rcntrc.cr_cash_acct_no
            let p_rcashe.cash_department = "000"
          when scr_fld = "rcpt_amt"
            let p_rcashe.rcpt_amt = round( "a", p_rcashe.rcpt_amt )
            let p_rcashe.bal_remaining = p_rcashe.bal_remaining -
                prev_data + p_rcashe.rcpt_amt
          when scr_fld = "auto_pay"
            if p_rcashe.auto_pay = "Y"
            then
                call automatic_pay()
                let p_rcashe.auto_pay = null
            end if
          when scr_fld = "cash_acct"
            # Perform lookups
            if llh_lookup("acct_look",true) = false and
              length(this_data) != 0
            then
                let nxt_fld = "cash_acct"
                return
            end if
          when scr_fld = "cash_department"
            # Perform lookups
            if llh_lookup("dept_look",true) = false and
              length(this_data) != 0
            then
                let nxt_fld = "cash_department"
                return
            end if
        end case
    end if

end function
# llh_a_field()

######################################################################
function llh_a_input()
######################################################################
# This function is called whenever the input statement exits
# (except due to an interrupt).  If you don't want the input session
# to end, set the nxt_fld variable to contain the field to be placed
# back into.
#
    if mustfill(p_rcashe.cust_code,"Customer Code")
      then let nxt_fld = "cust_code" return end if
    if mustfill(p_rcashe.rcpt_date,"Receipt Date")
      then let nxt_fld = "rcpt_date" return end if
    if mustfill(p_rcashe.cash_acct,"Cash Account Number")
      then let nxt_fld = "cash_acct" return end if
    if mustfill(p_rcashe.cash_department,"Cash Department Code")
      then let nxt_fld = "cash_department" return end if
    if tab_pressed = false
      then call cust_unlock() end if

end function
# llh_a_input()

######################################################################
function llh_event()
######################################################################
# This function is called whenever the user presses an event key.
# The event is mapped to the 'scr_funct' variable and processed here.
#
    # Reset tab pressed to false
    let tab_pressed = false
    case
      when scr_funct = "zoom" and infield(cust_code)
        if zoom("custz","")
        then
            let p_rcashe.cust_code = scratch
            let nxt_fld = "cust_code"
        end if
      when scr_funct = "zoom" and infield(cash_acct)
        let scratch =
          " stxchrtr.acct_type = \"A\""
        if zoom("chrtz","(see scratch)")
        then
            let p_rcashe.cash_acct = scratch
            let nxt_fld = "cash_acct"
        end if
      when scr_funct = "zoom" and infield(cash_department)
        let scratch =
          " stxinfor.src_type = \"D\""
        if zoom("deptz","(see scratch)")
        then
            let p_rcashe.cash_department = scratch
            let nxt_fld = "cash_department"
        end if
      when scr_funct = "cancel"
        call cust_unlock()
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
      when scr_funct = "accept"
        let nxt_fld = "exit input"
      when scr_funct = "tab" or scr_funct = "btab"
        let tab_pressed = true
        let nxt_fld = "exit input"
      when scr_funct = "cancel"
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
    end case
    let scr_funct = ""

end function
# llh_event()

######################################################################
function llh_setdata()
######################################################################
# This function is called to set the this_data global variable.
#
    case
      when scr_fld = "rcpt_date"
        call set_data(p_rcashe.rcpt_date)
      when scr_fld = "cust_code"
        call set_data(p_rcashe.cust_code)
      when scr_fld = "check_no"
        call set_data(p_rcashe.check_no)
      when scr_fld = "doc_desc"
        call set_data(p_rcashe.doc_desc)
      when scr_fld = "rcpt_amt"
        call set_data(p_rcashe.rcpt_amt)
      when scr_fld = "auto_pay"
        call set_data(p_rcashe.auto_pay)
      when scr_fld = "cash_acct"
        call set_data(p_rcashe.cash_acct)
      when scr_fld = "cash_department"
        call set_data(p_rcashe.cash_department)
    end case
end function
# llh_setdata()

######################################################################
function llh_high()
######################################################################
# This function highlights the specified field name.
# Only input type fields need to be specified.
#
    case
      when scr_fld = "rcpt_date"
        display by name p_rcashe.rcpt_date attribute(reverse)
      when scr_fld = "cust_code"
        display by name p_rcashe.cust_code attribute(reverse)
      when scr_fld = "check_no"
        display by name p_rcashe.check_no attribute(reverse)
      when scr_fld = "doc_desc"
        display by name p_rcashe.doc_desc attribute(reverse)
      when scr_fld = "rcpt_amt"
        display by name p_rcashe.rcpt_amt attribute(reverse)
      when scr_fld = "auto_pay"
        display by name p_rcashe.auto_pay attribute(reverse)
      when scr_fld = "cash_acct"
        display by name p_rcashe.cash_acct attribute(reverse)
      when scr_fld = "cash_department"
        display by name p_rcashe.cash_department attribute(reverse)
    end case
end function
# llh_high()

######################################################################
function llh_display()
######################################################################
# This function displays all p_* variables onto the screen
#
    display p_rcashe.* to s_rcashe.*  attribute(red)
end function
# llh_display()

######################################################################
function llh_p_prep()
######################################################################
# This function creates the p_* record from the m_* record
#
    let p_rcashe.rcpt_date = m_rcashe.rcpt_date
    let p_rcashe.cust_code = m_rcashe.cust_code
    let p_rcashe.check_no = m_rcashe.check_no
    let p_rcashe.doc_desc = m_rcashe.doc_desc
    let p_rcashe.cash_acct = m_rcashe.cash_acct
    let p_rcashe.cash_department = m_rcashe.cash_department

    # Perform lookups
    if llh_lookup("cust_look",false) then end if
    if llh_lookup("acct_look",false) then end if
    if llh_lookup("dept_look",false) then end if

    let p_rcashe.rcpt_amt = m_rcashe.cash_amt
    if m_rcashe.cash_deb_cred = "CR"
    then
        let p_rcashe.rcpt_amt = m_rcashe.cash_amt * -1
    end if
    let oa_amt = m_rcashe.oa_amt
    if m_rcashe.oa_deb_cred = "DB"
    then
        let oa_amt = m_rcashe.oa_amt * -1
    end if
    let p_rcashe.cash_acct = m_rcashe.cash_acct
    let p_rcashe.cash_department = m_rcashe.cash_department
    let oa_acct = m_rcashe.oa_acct
    let oa_department = m_rcashe.oa_department
    # if other cash receipts with same customer code
    # then recalculate the p_rcashe.on_acct_amt variable
    if dup_cust()
    then
        call calc_on_acct()
    end if
    let p_rcashe.paid_tot = p_rcashe.rcpt_amt - oa_amt
    let p_rcashe.bal_remaining =
       (p_rcashe.rcpt_amt - p_rcashe.paid_tot - p_rcashe.on_acct_amt)
    # general defaults
    if p_rcashe.rcpt_date is null
      then let p_rcashe.rcpt_date = today end if
    if p_rcashe.rcpt_amt is null
      then let p_rcashe.rcpt_amt = 0 end if
    if p_rcashe.on_acct_amt is null
      then let p_rcashe.on_acct_amt = 0 end if
    if oa_amt is null
      then let oa_amt = 0 end if
    if p_rcashe.cash_acct is null
      then let p_rcashe.cash_acct = m_rcntrc.cr_cash_acct_no end if
    if p_rcashe.cash_department is null
      then let p_rcashe.cash_department = "000" end if
    if p_rcashe.rcpt_amt is null
      then let p_rcashe.rcpt_amt = 0 end if
    if p_rcashe.disc_tot is null
      then let p_rcashe.disc_tot = 0 end if
    if p_rcashe.paid_tot is null
      then let p_rcashe.paid_tot = 0 end if
    if p_rcashe.bal_remaining is null
      then let p_rcashe.bal_remaining = 0 end if
    let p_rcashe.auto_pay = " "

end function
# llh_p_prep()

######################################################################
function llh_m_prep()
######################################################################
# This function creates the m_* record from the p_* record
#
    let m_rcashe.rcpt_date = p_rcashe.rcpt_date
    let m_rcashe.cust_code = p_rcashe.cust_code
    let m_rcashe.check_no = p_rcashe.check_no
    let m_rcashe.doc_desc = p_rcashe.doc_desc
    let m_rcashe.cash_acct = p_rcashe.cash_acct
    let m_rcashe.cash_department = p_rcashe.cash_department
    let oa_amt = p_rcashe.rcpt_amt - p_rcashe.paid_tot +
        p_rcashe.disc_tot

    let m_rcashe.cash_amt = p_rcashe.rcpt_amt
    let m_rcashe.cash_deb_cred = "DB"
    if p_rcashe.rcpt_amt < 0
    then
        let m_rcashe.cash_amt = p_rcashe.rcpt_amt * -1
        let m_rcashe.cash_deb_cred = "CR"
    end if
    let m_rcashe.oa_amt = oa_amt
    let m_rcashe.oa_deb_cred = "CR"
    if oa_amt < 0
    then
        let m_rcashe.oa_amt = oa_amt * -1
        let m_rcashe.oa_deb_cred = "DB"
    end if
    let m_rcashe.oa_acct = oa_acct
    let m_rcashe.oa_department = oa_department
    let m_rcashe.ok_to_post = "N"

end function
# llh_m_prep()

######################################################################
function llh_read(ring_rowid)
######################################################################
# This function loads the header p_* record elements.
# It first loads the m_* record from disk, then it
# calls the p_prep routine to fill the p_* record elements.
#
    define
        ring_rowid  integer     # rowid of record to read

    # Prepare the cursor upon first invocation
    if select_prep is null
    then
        let select_prep = "Y"
        let scratch = "select * from strcashe where rowid = ?"
        prepare read_specs from scratch
        declare h_cursor cursor for read_specs
    end if

    # Read the header record
    open h_cursor using ring_rowid
    fetch h_cursor into m_rcashe.*
    if sqlca.sqlcode != 0 then return end if
    let q_rcashe.row_id = ring_rowid
    close h_cursor
    call llh_p_prep()
end function
# llh_read()

######################################################################
function llh_delete(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    delete from strcashe
        where rowid = ring_rowid
end function
# llh_delete()

######################################################################
function llh_add()
######################################################################
# This function inserts data into the header table.
#
    define
        new_rowid integer  # Rowid after insert
    # Set the serial field
    let m_rcashe.doc_no = 0

    # Insert the data
    insert into strcashe values(m_rcashe.*)
    let new_rowid = sqlca.sqlerrd[6]

    # Bring back the serial field & display it
    let m_rcashe.doc_no = sqlca.sqlerrd[2]

    # Reset rowid
    let sqlca.sqlerrd[6] = new_rowid

end function
# llh_add()

######################################################################
function llh_update(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    update strcashe set strcashe.* = m_rcashe.*
      where rowid = ring_rowid
end function
# llh_update()


######################################################################
function llh_lookup(tbl_name,mustfind)
# returning false if notfound
######################################################################
# This is the lookup function.  It is passed a table to lookup, and
# based on that, it looks up the key and fills any variables for
# that table.  The lookup function will produce the error if it is
# called with mustfind=true, and a notfound situation exists.
#
    define
        tbl_name char(18),  # Lookup table name
        stat integer,       # Value of sqlca.sqlcode after lookup
        mustfind smallint   # (boolean) produce an error if notfound?

    # Prepare cursors upon first call to this function
    if lookup_prep is null
    then
        let lookup_prep = "Y"
        let scratch = "select bus_name, on_acct_amt ",
          "from strcustr where ",
          " strcustr.cust_code = ?"
        prepare str_cust_look from scratch
        declare cur_cust_look cursor for str_cust_look
        let scratch = "select rowid ",
          "from stxchrtr where stxchrtr.acct_no = ?"
        prepare str_acct_look from scratch
        declare cur_acct_look cursor for str_acct_look
        let scratch = "select rowid ",
          "from stxinfor where ",
          " stxinfor.src_type = \"D\" and stxinfor.src_key = ?"
        prepare str_dept_look from scratch
        declare cur_dept_look cursor for str_dept_look
    end if

    # Perform the lookup
    case
      when tbl_name = "cust_look"
        open cur_cust_look using p_rcashe.cust_code
        fetch cur_cust_look into p_rcashe.bus_name,
          p_rcashe.on_acct_amt
        let stat = sqlca.sqlcode
        close cur_cust_look
      when tbl_name = "acct_look"
        open cur_acct_look using p_rcashe.cash_acct
        fetch cur_acct_look into scratch
        let stat = sqlca.sqlcode
        close cur_acct_look
      when tbl_name = "dept_look"
        open cur_dept_look using p_rcashe.cash_department
        fetch cur_dept_look into scratch
        let stat = sqlca.sqlcode
        close cur_dept_look
    end case

    # No mustfind is this_data is empty
    if length(this_data) = 0 then let mustfind = false end if

    # Send notfound message to user (if requested)
    let sqlca.sqlcode = stat
    if mustfind and stat = notfound
      then call scr_error("lookup=notfound",tbl_name)
    end if
    let sqlca.sqlcode = 0

    # Return false if lookup failed and empty into fields
    if stat = notfound
    then
        case
          when tbl_name = "acct_look" or tbl_name = "dept_look"
            initialize scratch to null
          when tbl_name = "cust_look"
            initialize p_rcashe.bus_name to null
            initialize p_rcashe.on_acct_amt to null
        end case
        return false
    else
        return true
    end if
end function
# llh_lookup()
