######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  %Z%  %M%  %I%  Delta: %G%
######################################################################

globals "globals.4gl"

######################################################################
function mlh_init()
# returning the initial find group filter string.
######################################################################
# This function serves two purposes.
# It is called only once upon program invocation, so you may use
# it as a place to run initialization routines.
# It requires a return value of the default select group.
# examples of return values:
#   return "1=1" (select all rows)
#   return "1=0" (select no rows)
#   return "customer_num is not null" (select non-null customer_num's)
#   return "posted = \"N\"" (select only unposted documents)
#
    # Trap fatal errors
    whenever error call error_handler

    return "1=0"
end function
# mlh_init()

######################################################################
function mlh_clear()
######################################################################
#
    initialize p_rcashe.* to null
    initialize q_rcashe.* to null
    initialize m_rcashe.* to null
end function
# mlh_clear

######################################################################
function mld_clear()
######################################################################
#
    define
        n       smallint            # working number

    initialize m_rcashd.* to null    # clear the m_record
    initialize p_rcashd[1].* to null # clear the first p_record
    initialize q_rcashd[1].* to null # clear the first p_record
    for n = 2 to rec1_cnt
        let p_rcashd[n].* = p_rcashd[1].* # clear subsequent p_records
        let q_rcashd[n].* = q_rcashd[1].* # clear subsequent q_records
    end for
    let rec1_cnt = 0                # reset the active element counter
end function
# mld_mclear()

######################################################################
function mlh_construct()
# returning criteria
######################################################################
#
    construct scratch
      on
        strcashe.rcpt_date, strcashe.cust_code, strcashe.check_no,
        strcashe.doc_desc, strcashe.cash_acct, strcashe.cash_department
      from
        s_rcashe.rcpt_date, s_rcashe.cust_code, s_rcashe.check_no,
        s_rcashe.doc_desc, s_rcashe.cash_acct, s_rcashe.cash_department
    return scratch[1,512]
end function
# mlh_construct

######################################################################
function mlh_cursor()
######################################################################
# This function defines the table, filter, and ordering portions of
# the select statement used to build the FourGen scroller.
#
    call put_vararg("strcashe")
    call put_vararg("strcashe.cust_code is not null")
    call put_vararg("")
end function
# mlh_cursor()

######################################################################
function mld_scroll()
######################################################################
# This function calls display array for the scrolling input.
#
    if rec1_cnt < 1 then return end if
    call set_count(rec1_cnt)
    display array p_rcashd to s_rcashd.* attribute(red)
      on key (control-m,control-i)
        exit display
    end display
end function
# mld_scroll

######################################################################
function mld_arr_count()
######################################################################
# This function sets rec1_cnt to the correct number of array elements.
#
    define
        n smallint
    for n = arr_count() to 1 step -1
        if  p_rcashd[n].pay_method is not null
         or p_rcashd[n].inv_no is not null
         or p_rcashd[n].due_date is not null
         or p_rcashd[n].disc_date is not null
         or p_rcashd[n].disc_bal is not null
         or p_rcashd[n].balance is not null
         or p_rcashd[n].disc_amt is not null
         or p_rcashd[n].paid is not null
         or q_rcashd[n].row_id is not null
         or q_rcashd[n].inv_doc_no is not null
         or q_rcashd[n].dist_acct is not null
         or q_rcashd[n].dist_department is not null
        then
            let rec1_cnt = n
            return
        end if
    end for
    let rec1_cnt = 0
end function
# mld_arr_count()

######################################################################
function mlh_key()
# returning tablename, key_field [,key_field]... in (vararg) scratch
######################################################################
# This function returns (in vararg scratch) the name of the main table
# for this screen and the field name(s) and their data that build
# the unique key into this table.
# This is used to determine the unique key for additional fields,
# notes, and other items that may key off this document.  The key
# field(s) must not exceed 30 characters total.
# The first vararg call is the table name.  The following varargs are
# called in pairs.  The first of the pair is the column name, and
# the second in the pair is the column data.
#
    call put_vararg("strcashe")
    call put_vararg("doc_no")
    call put_vararg(m_rcashe.doc_no)
end function
# mlh_key()
