########################################################################
# Copyright (C) 1988,1989,1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ar.4gm/p_delete.4gs/lowlevel.4gl  1.11  Delta: 2/11/91
########################################################################

globals "globals.4gl"

define
    p_arc_date date     # archive date


#######################################################################
function before_group(group_key)
#######################################################################
#
    define
        group_key   char(20)    # group identification

    if group_key = "first_row" then
        # insert "on first row" code here
        return
    end if

end function
# before_group()

#######################################################################
function on_every_row()
#######################################################################
#  this function prepares the report record from the cursor record
#  and other data.
#
    define
        a_key char(1)

    let rpt.co_name = curs.co_name

    # get archive date
    let p_arc_date = get_arc_date()
    call sel_wind()

    begin work

    display " Locking Tables ..." at 4,1

        # lock stxtranr, strtranr, stractvd, stropend tables
        whenever error continue
        lock table stxtranr in share mode
        if sqlca.sqlcode != 0
        then
            call fg_err(4) # Could not lock stxtranr
            rollback work
            exit program(1)
        end if
        lock table strtranr in share mode
        if sqlca.sqlcode != 0
        then
            call fg_err(5) # Could not lock strtranr
            rollback work
            exit program(1)
        end if
        lock table stropend in share mode
        if sqlca.sqlcode != 0
        then
            call fg_err(6) # Could not lock stropend
            rollback work
            exit program(1)
        end if
        lock table stractvd in share mode
        if sqlca.sqlcode != 0
        then
            call fg_err(7) # Could not lock stractvd
            rollback work
            exit program(1)
        end if
        lock table strcustr in share mode
        if sqlca.sqlcode != 0
        then
            call fg_err(8) # Could not strcustr 
            rollback work
            exit program(1)
        end if
        whenever error call error_handler

        # delete stractvd
        call del_actvd()

        # delete strtranr
        call del_rtranr()

        display " Removing Header Activities ..." at 13,1
        # delete stxtranr
        call del_trx(p_arc_date)

    commit work

    close window selwin
    return
end function
# on_every_row()

#######################################################################
function after_group(group_key)
#######################################################################
#
    define
        group_key   char(20)    # group identification

    if group_key = "last_row" then
        # insert "on last row" code here
        return
    end if

end function
# after_group()

########################################################################
function del_actvd()
########################################################################
#
# delete activtiy that have zero balance and 
#   document date <= archive date
#
# delete on account activity with document date < archive date
#
    define
        customer_code like strcustr.cust_code, 
        p_amount like stractvd.amount,
        p_inv_doc_no like stractvd.inv_doc_no,
        p_rowid integer,
        quote char(1)

    let quote = "\""

    display " Removing A/R Detail Activities ..." at 7,1

    # delete all activity that does not have an assoicated open item
    declare orphan_actvd cursor for 
        select rowid from stractvd where inv_doc_no != 0 and not exists
            (select doc_no from stropend where doc_no = stractvd.inv_doc_no)

    foreach orphan_actvd into p_rowid
        delete from stpactvd where rowid = p_rowid
    end foreach

    # delete zero balance activity before the given date 
    declare actvd_del cursor for 
        select stractvd.inv_doc_no 
          from stractvd, stxtranr 
            where
              stxtranr.orig_journal = stractvd.orig_journal and
              stxtranr.doc_no = stractvd.doc_no and
              stractvd.act_type = "A" and
              stractvd.inv_doc_no != 0
            group by inv_doc_no
            having max(doc_date) <= p_arc_date and
              sum(amount) = 0)

   foreach actvd_del into p_inv_doc_no
        delete from stropend where doc_no = p_inv_doc_no
        delete from stractvd where inv_doc_no = p_inv_doc_no
    end foreach

    # update customer archive balance with all on-account amounts that
    # are before a given date.
    let ct.sel_stmt = 
       "select sum(amount), ref_code ",
         "from stxtranr, stractvd ",
         "where ",
            "stractvd.orig_journal = stxtranr.orig_journal and ",
            "stractvd.doc_no = stxtranr.doc_no and ",
            "stxtranr.doc_date <= ", quote, p_arc_date, quote,
            " and stractvd.inv_doc_no = 0 and ",
            "stractvd.act_type = ", quote, "A", quote, 
         " group by ref_code"

    prepare sel_on_acct from ct.sel_stmt
    declare on_acct_cur cursor for sel_on_acct

    foreach on_acct_cur into p_amount, customer_code
        update strcustr set arch_bal = arch_bal + p_amount
            where cust_code = customer_code
    end foreach

    display " Removing A/R On Account Activities ..." at 9,1

    # remove all on account amounts before given date
    declare on_acct cursor for 
        select stractvd.rowid from stractvd, stxtranr 
            where
               stractvd.inv_doc_no = 0 and
               stractvd.orig_journal = stxtranr.orig_journal and
               stractvd.doc_no = stxtranr.doc_no and
               stxtranr.doc_date <= p_arc_date

    foreach on_acct into p_rowid
        delete from stractvd where rowid = p_rowid
    end foreach

end function 
# del_actvd()

########################################################################
function del_rtranr()
########################################################################
#
# delete all strtranr that do not have activity 
#
      display " Removing A/R Header Activities ..." at 11,1
      delete from strtranr
        where not exists
          (select rowid from stractvd 
             where
               stractvd.orig_journal = strtranr.orig_journal and 
               stractvd.doc_no = strtranr.doc_no)

end function
# del_rtranr()
